﻿using Sid.Utilities;
using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

namespace TimeWebServer
{
	class Program
	{
		static void ShowHelp()
		{
			Console.WriteLine("Starts a web server which listens on the specified port of the local machine. It outputs the current local-time of the machine when accessed via HTTP.");
			Console.WriteLine("Syntax: TimeWebServer -port [Port] -logfolder [LogFolder]");			
			Console.WriteLine("Port      - Port number on which the server will listen");
			Console.WriteLine("LogFolder - Folder where log file will be written");
		}

		public static string GetLocalIPAddress()
		{
			IPHostEntry host = Dns.GetHostEntry(Dns.GetHostName());

			foreach (IPAddress ip in host.AddressList)
			{
				if (ip.AddressFamily == AddressFamily.InterNetwork)
				{
					return ip.ToString();
				}
			}

			throw new Exception("Local IP Address Not Found!");
		}

		static void Main(string[] args)
		{
			CmdLine cmd = new CmdLine(args);

			if (cmd.IsFlagPresent("help") || !cmd.IsFlagPresent("port") || !cmd.IsFlagPresent("logfolder"))
			{
				ShowHelp();
				return;
			}
						
			int port;

			if (!int.TryParse(cmd["port"], out port))
			{
				Console.WriteLine("Invalid port number deteced. Please specify a valid integer for the port number and try again.");
				return;
			}

			string logFolder = cmd["logfolder"];
			if (Directory.Exists(logFolder))
			{
				Directory.CreateDirectory(logFolder);
			}

			Console.WriteLine("Port: {0}\nLogfolder: {1}", port, logFolder);

			string ipAddress = GetLocalIPAddress();

			string logFile = Path.Combine(logFolder, "TimeWebServer.log");
			string prefix = string.Format("http://{0}:{1}/sws/", ipAddress, port);

			SimpleWebServer sws = new SimpleWebServer(new string[] { prefix }, logFile);
			sws.OnRequest += HandleRequest;
			sws.Start();

			Console.WriteLine("Press <ENTER> to stop the web server or send a stop command from the browser at {0}", prefix + "stop");
			Console.ReadLine();
			
			sws.Stop();
		}

		static void HandleRequest(HttpListenerContext context, SimpleWebServer caller)
		{
			Console.WriteLine("Received request from [{0}] at [{1}]" , context.Request.RemoteEndPoint.Address,
				DateTime.Now);

			HttpListenerRequest request = context.Request;
			HttpListenerResponse response = context.Response;
			string responseString;

			if (request.RawUrl == "/sws/viewlog")
			{
				string logs = string.Empty;
				using (StreamReader rdr = File.OpenText(caller.Logfile))
				{
					string line = rdr.ReadLine();
					StringBuilder sb = new StringBuilder();	
					while (line != null)
					{
						sb.AppendFormat("{0}<br>", line);
						line = rdr.ReadLine();
					}
					logs = sb.ToString();
				}

				responseString = string.Format("<HTML><HEAD><STYLE>body {{ font-family: tahoma, arial; }}</STYLE></HEAD><BODY>{0}</BODY></HTML>", logs);
			}
			else if (request.RawUrl == "/sws/stop")
			{
				caller.Stop();
				responseString = string.Format("<HTML><HEAD><STYLE>body {{ font-family: tahoma, arial; }}</STYLE></HEAD><BODY>{0}</BODY></HTML>", "Server will stop");
			}
			else
			{
				// Construct a response.
				responseString = string.Format("<HTML><HEAD><STYLE>body {{ font-family: tahoma, arial; }}</STYLE></HEAD><BODY>{0}</BODY></HTML>", DateTime.Now.ToString());
			}
			byte[] buffer = System.Text.Encoding.UTF8.GetBytes(responseString);
			
			// Get a response stream and write the response to it.
			response.ContentLength64 = buffer.Length;
			System.IO.Stream output = response.OutputStream;
			output.Write(buffer, 0, buffer.Length);
			output.Close();
		}		
	}
}
