﻿using System;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Threading;

namespace TimeWebServer
{
	public delegate void RequestReceivedDelegate(HttpListenerContext context, SimpleWebServer caller);

	public class SimpleWebServer
	{
		public SimpleWebServer (string[] prefixes, string logfile)
		{
			m_prefixes = prefixes;
			m_logfile = logfile;
		}

		public event RequestReceivedDelegate OnRequest;

		public string Logfile
		{
			get { return m_logfile; }
		}

		public void Start()
		{
			if (!HttpListener.IsSupported)
			{
				throw new NotSupportedException("Windows XP SP2 or Server 2003 is required to use the HttpListener class");
			}

			// URI prefixes are required, for example "http://contoso.com:8080/index/".
			if (m_prefixes == null || m_prefixes.Length == 0) throw new InvalidOperationException("No prefixes have been specified");
			
			// Create a listener.
			m_httpListener = new HttpListener();
			WriteLog("Created HTTP listener");

			// Add the prefixes.
			foreach (string prefix in m_prefixes)
			{
				m_httpListener.Prefixes.Add(prefix);
				Console.WriteLine("Listening on {0}", prefix);
				WriteLog("Adding prefix {0}", prefix);
			}

			m_listenerThread = new Thread(() =>
			{
				while (m_stop == false)
				{
					m_httpListener.Start();
					try
					{
						HttpListenerContext context = m_httpListener.GetContext();
						WriteLog("Received request");
						if (OnRequest != null)
						{
							OnRequest(context, this);
							WriteLog("Processed request");
						}
					}
					catch(Exception e)
					{
						WriteLog(e.Message);
					}
				}				
				WriteLog("Stopped listening for requests");
			});

			m_listenerThread.Start();
		}

		public void Stop()
		{
			m_stop = true;
			m_httpListener.Stop();
			Process.GetCurrentProcess().Kill();
		}

		private void WriteLog(string format, params object[] args)
		{
			lock (m_logLock)
			{
				using (TextWriter logWriter = new StreamWriter(m_logfile, true))
				{
					string line = string.Format(format, args);
					logWriter.WriteLine("{0} {1} - {2}", DateTime.Now.ToShortDateString(),
					DateTime.Now.ToLongTimeString(), line);
				}
			}
		}

		private string m_logfile = null;
		
		private string[] m_prefixes = null;
		private volatile bool m_stop = false;
		private object m_logLock = new object();
		private Thread m_listenerThread;
		private HttpListener m_httpListener = null;
	}
}
